@@ -3,13 +3,13 @@ import asyncio.events as events
 import os
 import sys
 import threading
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 from heapq import heappop
 
 
 def apply(loop=None):
     """Patch asyncio to make its event loop reentrant."""
-    loop = loop or asyncio.get_event_loop()
+    loop = loop or events._get_running_loop()
     if not isinstance(loop, asyncio.BaseEventLoop):
         raise ValueError('Can\'t patch loop of type %s' % type(loop))
     if getattr(loop, '_nest_patched', None):
@@ -26,10 +26,21 @@ def _patch_asyncio():
     Patch asyncio module to use pure Python tasks and futures,
     use module level _current_tasks, all_tasks and patch run method.
     """
-    def run(future, *, debug=False):
-        loop = asyncio.get_event_loop()
+    def run(main, *, debug=False):
+        loop = events._get_running_loop()
+        if not loop:
+            loop = events.new_event_loop()
+            events.set_event_loop(loop)
+            _patch_loop(loop)
         loop.set_debug(debug)
-        return loop.run_until_complete(future)
+        task = asyncio.ensure_future(main)
+        try:
+            return loop.run_until_complete(task)
+        finally:
+            if not task.done():
+                task.cancel()
+                with suppress(asyncio.CancelledError):
+                    loop.run_until_complete(task)
 
     if sys.version_info >= (3, 6, 0):
         asyncio.Task = asyncio.tasks._CTask = asyncio.tasks.Task = \
@@ -37,8 +48,8 @@ def _patch_asyncio():
         asyncio.Future = asyncio.futures._CFuture = asyncio.futures.Future = \
             asyncio.futures._PyFuture
     if sys.version_info < (3, 7, 0):
-        asyncio.tasks._current_tasks = asyncio.tasks.Task._current_tasks  # noqa
-        asyncio.all_tasks = asyncio.tasks.Task.all_tasks  # noqa
+        asyncio.tasks._current_tasks = asyncio.tasks.Task._current_tasks
+        asyncio.all_tasks = asyncio.tasks.Task.all_tasks
     if not hasattr(asyncio, '_run_orig'):
         asyncio._run_orig = getattr(asyncio, 'run', None)
         asyncio.run = run
@@ -74,7 +85,6 @@ def _patch_loop(loop):
         Simplified re-implementation of asyncio's _run_once that
         runs handles as they become ready.
         """
-        now = self.time()
         ready = self._ready
         scheduled = self._scheduled
         while scheduled and scheduled[0]._cancelled:
@@ -82,7 +92,8 @@ def _patch_loop(loop):
 
         timeout = (
             0 if ready or self._stopping
-            else min(max(scheduled[0]._when - now, 0), 86400) if scheduled
+            else min(max(
+                scheduled[0]._when - self.time(), 0), 86400) if scheduled
             else None)
         event_list = self._selector.select(timeout)
         self._process_events(event_list)
@@ -129,6 +140,9 @@ def _patch_loop(loop):
 
     @contextmanager
     def manage_asyncgens(self):
+        if not hasattr(sys, 'get_asyncgen_hooks'):
+            # Python version is too old.
+            return
         old_agen_hooks = sys.get_asyncgen_hooks()
         try:
             self._set_coroutine_origin_tracking(self._debug)
@@ -136,6 +150,7 @@ def _patch_loop(loop):
                 sys.set_asyncgen_hooks(
                     firstiter=self._asyncgen_firstiter_hook,
                     finalizer=self._asyncgen_finalizer_hook)
+            yield
         finally:
             self._set_coroutine_origin_tracking(False)
             if self._asyncgens is not None:
@@ -161,17 +176,6 @@ def _patch_loop(loop):
 
 def _patch_task():
     """Patch the Task's step and enter/leave methods to make it reentrant."""
-
-    def step(task, exc=None):
-        curr_task = curr_tasks.get(task._loop)
-        try:
-            step_orig(task, exc)
-        finally:
-            if curr_task is None:
-                curr_tasks.pop(task._loop, None)
-            else:
-                curr_tasks[task._loop] = curr_task
-
     Task = asyncio.Task
     if sys.version_info >= (3, 7, 0):
 
@@ -184,9 +188,18 @@ def _patch_task():
         asyncio.tasks._enter_task = enter_task
         asyncio.tasks._leave_task = leave_task
         curr_tasks = asyncio.tasks._current_tasks
-        step_orig = Task._Task__step
-        Task._Task__step = step
     else:
+
+        def step(task, exc=None):
+            curr_task = curr_tasks.get(task._loop)
+            try:
+                step_orig(task, exc)
+            finally:
+                if curr_task is None:
+                    curr_tasks.pop(task._loop, None)
+                else:
+                    curr_tasks[task._loop] = curr_task
+
         curr_tasks = Task._current_tasks
         step_orig = Task._step
         Task._step = step
